@@ -9,8 +9,9 @@ use Digest::MD5 qw(md5_hex);
 use Monitoring::Livestatus::INET;
 use Monitoring::Livestatus::UNIX;
 use Monitoring::Livestatus::MULTI;
+use Encode;
 
-our $VERSION = '0.46';
+our $VERSION = '0.54';
 
 
 =head1 NAME
@@ -126,27 +127,29 @@ sub new {
     my(%options) = @_;
 
     my $self = {
-      "verbose"                   => 0,       # enable verbose output
-      "socket"                    => undef,   # use unix sockets
-      "server"                    => undef,   # use tcp connections
-      "peer"                      => undef,   # use for socket / server connections
-      "name"                      => undef,   # human readable name
-      "line_seperator"            => 10,      # defaults to newline
-      "column_seperator"          => 0,       # defaults to null byte
-      "list_seperator"            => 44,      # defaults to comma
-      "host_service_seperator"    => 124,     # defaults to pipe
-      "keepalive"                 => 0,       # enable keepalive?
-      "errors_are_fatal"          => 1,       # die on errors
-      "backend"                   => undef,   # should be keept undef, used internally
-      "timeout"                   => undef,   # timeout for tcp connections
-      "query_timeout"             => 60,      # query timeout for tcp connections
-      "connect_timeout"           => 5,       # connect timeout for tcp connections
-      "timeout"                   => undef,   # timeout for tcp connections
-      "use_threads"               => undef,   # use threads, default is to use threads where available
-      "warnings"                  => 1,       # show warnings, for example on querys without Column: Header
-      "logger"                    => undef,   # logger object used for statistical informations and errors / warnings
-      "deepcopy"                  => undef,   # copy result set to avoid errors with tied structures
-      "disabled"                  => 0,       # if disabled, this peer will not receive any query
+      "verbose"                     => 0,       # enable verbose output
+      "socket"                      => undef,   # use unix sockets
+      "server"                      => undef,   # use tcp connections
+      "peer"                        => undef,   # use for socket / server connections
+      "name"                        => undef,   # human readable name
+      "line_seperator"              => 10,      # defaults to newline
+      "column_seperator"            => 0,       # defaults to null byte
+      "list_seperator"              => 44,      # defaults to comma
+      "host_service_seperator"      => 124,     # defaults to pipe
+      "keepalive"                   => 0,       # enable keepalive?
+      "errors_are_fatal"            => 1,       # die on errors
+      "backend"                     => undef,   # should be keept undef, used internally
+      "timeout"                     => undef,   # timeout for tcp connections
+      "query_timeout"               => 60,      # query timeout for tcp connections
+      "connect_timeout"             => 5,       # connect timeout for tcp connections
+      "timeout"                     => undef,   # timeout for tcp connections
+      "use_threads"                 => undef,   # use threads, default is to use threads where available
+      "warnings"                    => 1,       # show warnings, for example on querys without Column: Header
+      "logger"                      => undef,   # logger object used for statistical informations and errors / warnings
+      "deepcopy"                    => undef,   # copy result set to avoid errors with tied structures
+      "disabled"                    => 0,       # if disabled, this peer will not receive any query
+      "retries_on_connection_error" => 3,       # retry x times to connect
+      "retry_interval"              => 1,       # retry after x seconds
     };
 
     for my $opt_key (keys %options) {
@@ -158,6 +161,11 @@ sub new {
         }
     }
 
+    if($self->{'verbose'} and !defined $self->{'logger'}) {
+        croak('please specify a logger object when using verbose mode');
+        $self->{'verbose'} = 0;
+    }
+
     # setting a general timeout?
     if(defined $self->{'timeout'}) {
         $self->{'query_timeout'}   = $self->{'timeout'};
@@ -200,7 +208,7 @@ sub new {
         $self->{'peer'} = $self->{'CONNECTOR'}->{'peer'};
     }
 
-    if(defined $self->{'logger'} and (!defined $self->{'backend'} or $self->{'backend'} ne 'Monitoring::Livestatus::MULTI')) {
+    if($self->{'verbose'} and (!defined $self->{'backend'} or $self->{'backend'} ne 'Monitoring::Livestatus::MULTI')) {
         $self->{'logger'}->debug('initialized Monitoring::Livestatus ('.$self->peer_name.')');
     }
 
@@ -281,9 +289,7 @@ sub selectall_arrayref {
     # make opt hash keys lowercase
     $opt = $self->_lowercase_and_verify_options($opt);
 
-    if(defined $self->{'logger'}) {
-        $self->_log_statement($statement, $opt, $limit);
-    }
+    $self->_log_statement($statement, $opt, $limit) if $self->{'verbose'};
 
     $result = $self->_send($statement, $opt);
 
@@ -827,13 +833,16 @@ sub _send {
         }
         chomp($statement);
         my $send = "$statement\n$header";
-        print "> ".Dumper($send) if $self->{'verbose'};
+        $self->{'logger'}->debug("> ".Dumper($send)) if $self->{'verbose'};
         ($status,$msg,$body) = $self->_send_socket($send);
         if($self->{'verbose'}) {
-            print "status: ".Dumper($status);
-            print "msg:    ".Dumper($msg);
-            print "< ".Dumper($body);
+            #$self->{'logger'}->debug("got:");
+            #$self->{'logger'}->debug(Dumper(\@erg));
+            $self->{'logger'}->debug("status: ".Dumper($status));
+            $self->{'logger'}->debug("msg:    ".Dumper($msg));
+            $self->{'logger'}->debug("< ".Dumper($body));
         }
+        #($status,$msg,$body) = @erg;
     }
 
     if($status >= 300) {
@@ -845,7 +854,7 @@ sub _send {
         } else {
             $Monitoring::Livestatus::ErrorMessage = $msg;
         }
-        $self->{'logger'}->error($status." - ".$Monitoring::Livestatus::ErrorMessage." in query:\n'".$statement) if defined $self->{'logger'};
+        $self->{'logger'}->error($status." - ".$Monitoring::Livestatus::ErrorMessage." in query:\n'".$statement) if $self->{'verbose'};
         if($self->{'errors_are_fatal'}) {
             croak("ERROR ".$status." - ".$Monitoring::Livestatus::ErrorMessage." in query:\n'".$statement."'\n");
         }
@@ -869,6 +878,7 @@ sub _send {
     my $row_count = 0;
     ## no critic
     for my $line (split/$line_seperator/m, $body) {
+        utf8::decode($line);
         $row_count++;
         next unless $row_count >= $limit_start;
 
@@ -888,7 +898,7 @@ sub _send {
 
     # for querys with column header, no seperate columns will be returned
     if(!defined $keys) {
-        $self->{'logger'}->warn("got statement without Columns: header!") if defined $self->{'logger'};
+        $self->{'logger'}->warn("got statement without Columns: header!") if $self->{'verbose'};
         if($self->{'warnings'}) {
             carp("got statement without Columns: header! -> ".$statement);
         }
@@ -923,7 +933,8 @@ sub _open {
     my $statement = shift;
 
     # return the current socket in keep alive mode
-    if($self->{'keepalive'} and defined $self->{'sock'} and $self->{'sock'}->atmark()) {
+    if($self->{'keepalive'} and defined $self->{'sock'} and $self->{'sock'}->connected) {
+        $self->{'logger'}->debug("reusing old connection") if $self->{'verbose'};
         return($self->{'sock'});
     }
 
@@ -934,6 +945,7 @@ sub _open {
         $self->{'sock'} = $sock;
     }
 
+    $self->{'logger'}->debug("using new connection") if $self->{'verbose'};
     return($sock);
 }
 
@@ -941,6 +953,7 @@ sub _open {
 sub _close {
     my $self  = shift;
     my $sock  = shift;
+    undef $self->{'sock'};
     return($self->{'CONNECTOR'}->_close($sock));
 }
 
@@ -1043,27 +1056,75 @@ The values from all backends with be summed up to a total.
 
 
 ########################################
+# wrapper around _send_socket_do
 sub _send_socket {
     my $self      = shift;
     my $statement = shift;
+
+    my $retries = 0;
+    my($status, $msg, $recv);
+
+
+    # try to avoid connection errors
+    eval {
+        local $SIG{PIPE} = sub {
+            die("broken pipe");
+            $self->{'logger'}->debug("broken pipe, closing socket") if $self->{'verbose'};
+            $self->_close($self->{'sock'});
+        };
+
+        if($self->{'retries_on_connection_error'} <= 0) {
+            ($status, $msg, $recv) = $self->_send_socket_do($statement);
+            return;
+        }
+
+        while((!defined $status or ($status == 491 or $status == 497 or $status == 500)) and $retries < $self->{'retries_on_connection_error'}) {
+            $retries++;
+            ($status, $msg, $recv) = $self->_send_socket_do($statement);
+            $self->{'logger'}->debug('query status '.$status) if $self->{'verbose'};
+            if($status == 491 or $status == 497 or $status == 500) {
+                $self->{'logger'}->debug('got status '.$status.' retrying in '.$self->{'retry_interval'}.' seconds') if $self->{'verbose'};
+                $self->_close();
+                sleep($self->{'retry_interval'}) if $retries < $self->{'retries_on_connection_error'};
+            }
+        }
+    };
+    if($@) {
+        $self->{'logger'}->debug("try 1 failed: $@") if $self->{'verbose'};
+        if(defined $@ and $@ =~ /broken\ pipe/mx) {
+            return $self->_send_socket_do($statement);
+        }
+        croak($@) if $self->{'errors_are_fatal'};
+    }
+
+    croak($msg) if($status >= 400 and $self->{'errors_are_fatal'});
+
+    return($status, $msg, $recv);
+}
+
+########################################
+sub _send_socket_do {
+    my $self      = shift;
+    my $statement = shift;
     my($recv,$header);
 
     my $sock = $self->_open() or return(491, $self->_get_error(491), $!);
-    print $sock $statement or return($self->_socket_error($statement, $sock, 'connection failed: '.$!));;
+    utf8::decode($statement);
+    print $sock encode('utf-8' => $statement) or return($self->_socket_error($statement, $sock, 'write to socket failed: '.$!));
+
     if($self->{'keepalive'}) {
         print $sock "\n";
-    }else {
+    } else {
         $sock->shutdown(1) or croak("shutdown failed: $!");
     }
 
     # COMMAND statements never return something
     if($statement =~ m/^COMMAND/mx) {
-        $self->_close($sock);
         return('201', $self->_get_error(201), undef);
     }
 
-    $sock->read($header, 16) or return($self->_socket_error($statement, $sock, 'reading from socket failed: '.$!));
-    print "header: $header" if $self->{'verbose'};
+    $sock->read($header, 16) or return($self->_socket_error($statement, $sock, 'reading header from socket failed, check your livestatus logfile: '.$!));
+    $self->{'logger'}->debug("header: $header") if $self->{'verbose'};
     my($status, $msg, $content_length) = $self->_parse_header($header, $sock);
     return($status, $msg, undef) if !defined $content_length;
     if($content_length > 0) {
@@ -1086,13 +1147,17 @@ sub _socket_error {
     $message   .= "statement           ".Dumper($statement);
     $message   .= "message             ".Dumper($body);
 
-    $self->{'logger'}->error($message) if defined $self->{'logger'};
+    $self->{'logger'}->error($message) if $self->{'verbose'};
 
-    if($self->{'errors_are_fatal'}) {
-        croak($message);
-    } else {
-        carp($message);
+    if($self->{'retries_on_connection_error'} <= 0) {
+        if($self->{'errors_are_fatal'}) {
+            croak($message);
+        }
+        else {
+            carp($message);
+        }
     }
+    $self->_close();
     return(500, $self->_get_error(500), $message);
 }
 
@@ -1427,7 +1492,8 @@ sub _log_statement {
 
     my $cleanstatement = $statement;
     $cleanstatement =~ s/\n/\\n/gmx;
-    $self->{'logger'}->debug('selectall_arrayref("'.$cleanstatement.'", '.$optstring.', '.$limit.')')
+    $self->{'logger'}->debug('selectall_arrayref("'.$cleanstatement.'", '.$optstring.', '.$limit.')');
+    return 1;
 }
 
 ########################################
